package ac.essex.ca.neighbourhoods;

import ac.essex.ca.rules.Rule;

/**
 * <p>
 * Von Neumann's Neighbourhood, where a cell looks at the three cells
 * above it to decide what its state should be. This produces a triangular
 * shape. Cellular Automata using Von Neumann's neighbourhoods thus often have lots
 * of triangle shapes in them.
 * </p>
 *
 * <p>
 * <table border="1">
 * <tr><td>x</td><td>x</td><td>x</td></tr>
 * <tr><td>&nbsp;</td><td>p</td><td>&nbsp;</td>
 * </table>
 * </p>
 *
 * <p>
 * As the neighbourhood only looks at three cells, there are 8 different situations to look at:
 * </p>
 *
 * <p>
 * 111      110     101     100     011     010     001     000
 * </p>
 *
 * <p>
 * For each of the situations above, there the rule can set the pixel to be
 * either 1 or 0. This gives 256 possible rule sets.
 * </p>
 *
 * <p>
 * Von Neumann encoded the rule sets into a set of decimal values between 1 - 256. Binary readers
 * will be able to see the rule below is set number 30
 * </p>
 *
 * <p>
 * Pixels: 111      110     101     100     011     010     001     000
 * Output: 0        0       0       1       1       1       1       0
 * </p>
 *
 * <p>
 * Other neighbourhoods will have many more combinations of rule sets
 * </p>
 *
 * <p/>
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version,
 * provided that any use properly credits the author.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details at http://www.gnu.org
 * </p>
 *
 * @author Olly Oechsle, University of Essex, Date: 08-Aug-2006
 * @version 1.0
 */
public class VonNeumannNeighbourhood extends Neighbourhood {

    public VonNeumannNeighbourhood() {
        super();
    }

    /**
     * Creates the neighbourhood by specifying the location of the three cells
     * above the current position.
     *
     * If you change the locations of these three pixels, you can cause the CA
     * to produce different shapes.
     */
    public void createNeighbourhood() {
        addPosition(-1,-1);
        addPosition( 0,-1);
        addPosition( 1,-1);
    }

}
